const OP = require('./op.js');


function TokenStream(input){
  var pos = 0;

  function peek(){
    return (pos < input.length) ? input[pos] : null;
  }

  function next(){
    if (pos < input.length){
      let v = input[pos];
      pos += 1;
      return v;
    }
    return null;
  }

  function line(){
    return (pos < input.length) ? input[pos].line : -1;
  }

  function col(){
    return (pos < input.length) ? input[pos].col : -1;
  }

  function eof(){
    return (pos >= input.length);
  }

  function die(msg){
    throw new Error(msg + " Line: " + input[pos].line + ", Col: " + input[pos].col);
  }

  return {
    peek: peek,
    next: next,
    line: line,
    col: col,
    eof, eof,
    die: die
  };
}
var stream = null;

function isTokenType(type, val){
  let t = stream.peek();
  if (t !== null && t.type === type){
    if (!val || t.val === val)
      return true;
  }
  return false;
}

function isPunc(ch){
  return isTokenType("punk", ch);
}

function isOpCode(ch){
  return isTokenType("opcode", ch);
}

function isLabel(ch){
  return isTokenType("label", ch);
}

function isDirective(ch){
  return isTokenType("directive", ch);
}

function isOp(ch){
  return isTokenType("op", ch);
}



parse(tokens){
  let p = {
    type: "prog",
    expressions: [];
  };
  stream = TokenStream(tokens);
  while (!stream.eof()){
    p.expressions.push(parseExpression());
  }
  stream = null;
  return p;
}


module.exports = Object.freeze({
  parse: parse
});



